############################################################
#                       AUTH-EXTRA-HTML.PL
#
# Dieses Script wurde von Selena Sol & Gunther Birznieks geschrieben. 
# Urspruengliche Programmierung:	10.05.96
# Letzte Aenderung:			14.05.96
# Deutsche Uebersetzung: Gerold Bigorajski
#
#   Es darf unter den Bedingungen der GNU General Public
#   License oder der Artistic License, die zusammen mit
#   Perl v5.x fr UNIX verteilt wird, kopiert werden.
# 
# Selena Sol kann unter selena@eff.org kontaktiert werden.
# Gerold Bigorajski ist unter bigorajski@t-online.de erreichbar.
#
# Zweck: Dieses Script enthaelt die komplette kosmetische HTML-Ausgabe,
# die mit den Authentiserungsroutinen zusammenhaengt.
#
# Hauptprozeduren:
#   Alle Routinen sind Ergaenzungen zur Datei auth-extra-lib.pl. 
#
# Spezielle Anmerkungen: Beinahe alle unten aufgefuehrten Routinen
# akzeptieren irgendeine Art von Parameter. Diese Parameter dienen
# nur der Ausagabe zusaetzlicher Informationen. Sie bewirken keine
# signifikante Bearbeitung irgendeiner Art.
#
#
############################################################

############################################################
#
# Subroutine: PrintLogonPage
#
#  Diese Routine gibt die Login-HTML-Seite aus und ggf. eine
#  Meldung ueber fehlgeschlagenes Login, falls sie existiert.
#  Es werden automatsich verborgenen Felder erzeugt, falls
#  fruehere Formulardaten vom Hauptscript uebergeben werden.
#
############################################################

############################################################
#
# $bodytag ist der gemeinsame Bodytag fuer alle in dieser Bibliothek
# verwendeten HTML-Ausgaben.
# $seperatimg ist ein als Trennlinie verwendetes Bild oder aber einfach <hr>.
# Diese Variablen sollte im Variablenblock der aufrufenden Routine definiert
# werden. Hier stehen Sie in auth_lib.pl.
#
############################################################

sub PrintLogonPage
{
	local($bad_logon_message, $main_script, *in) = @_;
	local($form_tags);
	local($register_tag);
	local($search_tag);

	$register_tag = "";
	$search_tag = "";

	if ($auth_allow_register eq "on")
	{
		$register_tag = <<__END_OF_REG_TAG__;
<input type=submit name=auth_register_screen_op value="Benutzerregistrierung"><p>
__END_OF_REG_TAG__
}

	if ($auth_allow_search eq "on")
	{
		$search_tag = <<__END_OF_SEARCH_TAG__;
<input type=submit name=auth_search_screen_op value="Suche nach alter Registrierung"><p>
__END_OF_SEARCH_TAG__
	} 
	$form_tags = &PrintCurrentFormVars(*in);

	print <<__END_OF_LOGON__;
<HTML><HEAD>
<TITLE>Login</TITLE>
</HEAD>
$bodytag
<H2><FONT COLOR="#003366">Geben Sie Ihre Login-Informationen ein </FONT></H2>
<center>
$separatimg
$bad_logon_message
<FORM METHOD=POST ACTION=$main_script>
$form_tags
<TABLE>
	<TR>
		<TH>Benutzername</TH>
		<TD><INPUT TYPE="TEXT" NAME="auth_user_name"></td>
	</tr>
	<tr>
		<th>Kennwort</th>
		<td><input type="password" name="auth_password"></td>
	</tr>
</TABLE><p>
<input type="submit" name="auth_logon_op" value="System-Login"><p>
$register_tag
$search_tag
$separatimg
</center>
</form>
</body>
</HTML>
__END_OF_LOGON__

} # Ende der Subroutine PrintLogonPage

############################################################
#
# Subroutine: PrintSearchPage
#
#  Diese Routine gibt die HTML-Suchseite aus, falls eine existiert.
#  Es werden automatsich verborgenen Felder erzeugt, falls
#  fruehere Formulardaten vom Hauptscript uebergeben werden.
#
############################################################

sub PrintSearchPage
{
	local($main_script,*in) = @_;
	local($form_tags);
	$form_tags = &PrintCurrentFormVars(*in);
	print <<__END_OF_SEARCH__;
<HTML><HEAD>
<TITLE>Suche nach einem Benutzerkonto</TITLE>
</HEAD>
$bodytag
<H2><FONT COLOR="#003366">Suche nach bereinstimmendem Benutzernamen</FONT></H2>
$separatimg
<CENTER>
<H3><FONT COLOR="#FF0000">Geben Sie Ihre email-Adresse ein, um nach dem entsprechenden Benutzernamen zu suchen</FONT></H3>
<FORM METHOD=POST ACTION=$main_script>
$form_tags
<TABLE>
	<TR>
		<TH>Email-Adresse</TH>
		<TD><INPUT TYPE=TEXT NAME=auth_email></td>
	</TR>
</TABLE>
<p>
<input type="submit" name="auth_search_op" value="      Suchen      "> <input type="submit" name="auth_logon_screen_op" value="Zurck zum Login">
<P>
</center>
</form>
</body>
</HTML>
__END_OF_SEARCH__

} # End of PrintSearchPage

############################################################
#
# Subroutine: HTMLPrintSearchResults
#
#   Diese Routine gibt die Ergebnisse der Suche nach dem
#   Benutzernamen mittels der email-Adresse aus.
#
############################################################

sub HTMLPrintSearchResults
{
	local($main_script, $form_tags, $user_list) = @_;
	print <<__END_SEARCHRESULTS__;
<HTML><HEAD>
<TITLE>Benutzer gefunden</TITLE>
</HEAD>
$bodytag
<H2><FONT COLOR="#003366">Benutzername wurde gefunden</FONT></H2>
$separatimg
<CENTER>
<H3><FONT COLOR="#FF0000">Liste der Benutzer</FONT></H3>
<strong>$user_list</strong>
<FORM METHOD=POST ACTION=$main_script>
$form_tags
<input type="submit" name="auth_logon_screen_op" value="Zurck zum Login">
$separatimg
</center>
</form>
</body>
</HTML>
__END_SEARCHRESULTS__

} # Ende der Subroutine HTMLPrintSearchResults

############################################################
#
# Subroutine: HTMLPrintNoSearchResults
#
#   Diese Routine bewirkt die HTML-Ausgabe, falls die Suche mittels
#   der email-Adresse keinerlei Ergebnisse produziert.
#
############################################################

sub HTMLPrintNoSearchResults
{
	local($main_script, $form_tags) = @_;
	print <<__END_NOSEARCHRESULTS__;
<HTML><HEAD>
<TITLE>Benutzernamen nicht gefunden</TITLE>
</HEAD>
$bodytag
<H2><FONT COLOR="#003366">Der Benutzername wurde nicht gefunden</FONT></H2>
$separatimg
<CENTER>
<FORM METHOD=POST ACTION=$main_script>
$form_tags
<input type="submit" name="auth_logon_screen_op" value="Zurck zum Login">
$separatimg
</center>
</form>
</body>
</HTML>
__END_NOSEARCHRESULTS__

} # Ende der Subroutine HTMLPrintNoSearchResults


############################################################
#
# Subroutine: PrintRegisterPage
#
#  Diese Routine gibt die Registrierungseite aus.
#  Es werden automatsich verborgenen Felder erzeugt, falls
#  fruehere Formulardaten vom Hauptscript uebergeben werden.
#
############################################################

sub PrintRegisterPage
{
	local($main_script,*in) = @_;
	local($form_tags);
	local($more_form_input,$password_input, $x);
	$form_tags = &PrintCurrentFormVars(*in);
	local ($password_message);

	# Es findet auch eine Ueberpruefung in bezug auf die
	# zusaetzlichen Felder statt und ggf. eine HTML-Ausgabe,
	# die eine Eingabe in diese Felder verlangt.

	$more_form_input = "";
	for ($x = 0; $x <= @auth_extra_fields - 1; $x++)
	{
		$more_form_input .= <<__END_OF_EXTRA_FIELDS__;
	<TR>
		<TH>$auth_extra_desc[$x]</TH>
		<TD><INPUT TYPE="TEXT" NAME="$auth_extra_fields[$x]"></TD>
	</TR>
__END_OF_EXTRA_FIELDS__
	}
	$password_input = "";
	if ($auth_generate_password ne "on")
	{
		$password_input = <<__END_OF_PASSWORD_FIELDS__;
	<TR>
		<TH>Kennwort</TH>
		<td><input type="password" name="auth_password1"></TD>
	</TR>
	<TR>
		<TH>Kennwort wiederholen</TH>
		<TD><input type="password" name="auth_password2"></TD>
	</TR>
__END_OF_PASSWORD_FIELDS__
	} 
	$password_message = "";
	if ($auth_generate_password eq "on")
	{
		$password_message = <<__PASSWORDMSG__;
Ihr Kenwort wird automatisch erzeugt und an Ihre email-Adresse geschickt.
__PASSWORDMSG__
	} 
	print <<__END_OF_REGISTER__;
<HTML><HEAD>
<TITLE>Benutzerregistrierung</TITLE>
</HEAD>
$bodytag
<H2><FONT COLOR="#003366">Eingabe der Regsitrierungs-Informationen</FONT></H2>
$separatimg
<CENTER>
<FORM METHOD=POST ACTION=$main_script>
$form_tags
<TABLE>
	<tr>
		<th>Benutzername</th>
		<td><input type="Username" name="auth_user_name"></td>
	</tr>
	$password_input
	$more_form_input
</TABLE>
<p>
<input type="submit" name="auth_register_op" value="Informationen absenden">
<input type="submit" name="auth_logon_screen_op" value="   Zurck zum Login   ">
<P>
$password_message
</center>
</form>
</body>
</HTML>
__END_OF_REGISTER__

} # Ende der Subroutine PrintRegisterPage

############################################################
#
# Subroutine: HTMLPrintRegisterSuccess
#
#  Diese Routine druckt die HTML-Bestatigung nach erfolgreicher
#  Benutzer-Registrierung.
#
############################################################

sub HTMLPrintRegisterSuccess
{
	local($main_script, $form_tags) =  @_;
	print <<__END_OF_REGISTER_SUCCESS__;
<HTML><HEAD>
<TITLE>Registrierung aufgenommen</TITLE>
</HEAD>
$bodytag
<H2><FONT COLOR="#003366">Ihre Registrierung wurde der Datenbank hinzugefgt</FONT></H2>
$separatimg
<CENTER>
<FORM METHOD=POST ACTION=$main_script>
$form_tags
<BLOCKQUOTE>
    $auth_register_message
</Blockquote>
<input type="submit" name="auth_logon_screen_op" value="Zurck zum Login")
</center>
</form>
</body>
</HTML>
__END_OF_REGISTER_SUCCESS__

} # Ende der Subroutine RegisterSuccess

############################################################
#
# Subroutine: HTMLPrintRegisterFoundDuplicate
#
#  Diese Routine erzeugt die HTML-Ausgabe nach fehlgeschlagener
#  Benutzerregistrierung aufgrund eines doppelten
#  Benutzernamens.
#
############################################################

sub HTMLPrintRegisterFoundDuplicate
{
	local($main_script, $form_tags) = @_;
	print <<__END_OF_REGISTER_DUPLICATE__;
<HTML><HEAD>
<TITLE>Problem bei der Registrierung</TITLE>
</HEAD>
$bodytag
<H2><FONT COLOR="#003366">Problem bei der Registrierung</FONT></H2>
$separatimg
<CENTER>
<FORM METHOD=POST ACTION=$main_script>
$form_tags
<BLOCKQUOTE>
Der Benutzername ist in der Datenbank bereits vorhanden! Bitte geben Sie einen anderen ein.
</Blockquote>
<input type="submit" name="auth_logon_screen_op" value="Zurck zum Login")
</center>
</form>
</body>
</HTML>
__END_OF_REGISTER_DUPLICATE__

} # Ende der Subroutine HTMLPrintRegisterFoundDuplicate

############################################################
#
# Subroutine: HTMLPrintRegisterNoPasswordMatch
#
#  Diese Routine erzeugt die HTML-Ausgabe nach fehlgeschlagener
#  Benutzerregistrierung aufgrund nicht uebereinstimmender
#  Kennwoerter.
#
############################################################

sub HTMLPrintRegisterNoPasswordMatch
{
	local($main_script, $form_tags) = @_;
	print <<__END_OF_REGISTER_NOMATCH__;
<HTML><HEAD>
<TITLE>Problem bei der Registrierung</TITLE>
</HEAD>
$bodytag
<H2><FONT COLOR="#003366">Problem bei der Registrierung</FONT></H2>
$separatimg
<CENTER>
<FORM METHOD=POST ACTION=$main_script>
$form_tags
<BLOCKQUOTE>
Die beiden eingegebenen Kennwoerter stimmen nicht berein! Bitte versuchen Sie es noch einmal.
</Blockquote>
<input type="submit" name="auth_logon_screen_op" value="Zurck zum Login")
</center>
</form>
</body>
</HTML>
__END_OF_REGISTER_NOMATCH__

} # Ende der Subroutine HTMLPrintRegisterNoPasswordMatch

############################################################
#
# Subroutine: HTMLPrintRegisterFoundDuplicate
#
#  Diese Routine erzeugt die HTML-Ausgabe nach fehlgeschlagener
#  Benutzerregistrierung aufgrund eines fehlenden Feldwertes
#  oder wenn ein Feldwert den senkrechten Strich enthaelt.
#
############################################################

sub HTMLPrintRegisterNoValidValues
{
	local ($main_script, $form_tags) = @_;
	print <<__END_OF_REGISTER_NOVALUE__;
<HTML><HEAD>
<TITLE>Problem bei der Registrierung</TITLE>
</HEAD>
$bodytag
<H2><FONT COLOR="#003366">Problem bei der Registrierung</FONT></H2>
$separatimg
<CENTER>
<FORM METHOD=POST ACTION=$main_script>
$form_tags
<BLOCKQUOTE>
In allen Feldern ist eine gltige Eingabe erforderlich. Die Eingaben drfen 
<b>nicht</b> den senkrechten Querstrich ("|") enthalten.
</Blockquote>
<input type="submit" name="auth_logon_screen_op" value="Zurck zum Login")
</center>
</form>
</body>
</HTML>
__END_OF_REGISTER_NOVALUE__
 
} # Ende der Subroutine HTMLPrintRegisterNoValidValues

1;
