############################################################
#                       AUTH-EXTRA-LIB.PL
#
# Dieses Script wurde von Selena Sol & Gunther Birznieks geschrieben. 
# Urspruengliche Programmierung:	10.05.96
# Letzte Aenderung:			14.05.96
# Deutsche Uebersetzung: Gerold Bigorajski
#
#   Es darf unter den Bedingungen der GNU General Public
#   License oder der Artistic License, die zusammen mit
#   Perl v5.x fr UNIX verteilt wird, kopiert werden.
# 
# Selena Sol kann unter selena@eff.org kontaktiert werden.
# Gerold Bigorajski ist unter bigorajski@t-online.de erreichbar.
#
# Zweck: Stellt eine Reihe von Unterstuetzungsroutinen fuer die
# Bibliothek AUTH-LIB.PL zur Verfuegung.
#
# Hauptprozeduren:
#   VerifyUser -	Loggt den Benutzer ins System ein und registriert ihn.
#
# Spezielle Anmerkungen: Dieses Script verlaesst sich fuer den Versand von
# email (falls dieser Punkt aktiviert ist) auf die Bibliothek mail-lib.pl.
#
# Es verwendet auch CRYPT innerhalb der EncrytWrap-Routine. Falls Ihr
# Betriebssystem CRYPT nicht untersttzt, koennen Sie innerhalb von
# EncryptWrap Ihre eigene Verschluesselungsroutine schreiben.
#
############################################################

# Die kosmetische HTML-Ausgabe bleibt in einer Extra-Datei, die einfacher
# ausserhalb der mit dem Code verknuepften Logik modifziert werden kann.
#

require "$auth_lib/auth-extra-html.pl";

############################################################
#
# Subroutine: VerifyUser 
#   Verwendung:
#   ($session, $username, $group, @extra_fields
#    = &VerifyUser("script name", *in);
#
# Diese Routine ist das Kernstueck des ganzen. Sie arbeitet das
# gesamte Login und die Registrierungsinformation ab, wenn noch
# keine gueltige Sitzung zustande gekommen ist. Schliesslich
# gibt sie eine Sitzungs-ID zurueck, wenn der Benutzer sich
# erfolgreich eingeloggt hat.
#
#  Parameter:
#	$main_script = das Script, von dem aus &GetSessionInfo
#			   aufgerufen wird.
#	*in =	Ein Verweis auf die Formulardaten, die mit &ReadParse
#		eingelesen wurden.
#
#   Ausgabe:
#	$session  = Sitzungs-ID
#	$username = Benutzername
#	$group    = Gruppen-Information
#	@extra_fields = i.d.R. ein Array weiterer Felder,
#     die aus folgendem bestehen:
#       $first_name = Vorname
#       $last_name  = Nachname
#       $email      = email-Adresse
############################################################

sub VerifyUser
{
	local ($main_script, *in) = @_;
	local ($session, $group, $username, @extra_fields, $bad_logon_message);

	# Falls das vorherige Login gescheitert ist, enthaelt bad_logon_message
	# eine auszudruckende Mitteilung.

	$bad_logon_message = "";

	# Es gibt zwei Moeglichkeiten, einen Benutzer zu authentisieren:
	# Per CGI und server-baisert. CGI-Authentiserung verwendet diese
	# Programmroutinen und HTML-Formulare, um den Benutzer in das 
	# System einzuloggen. Serverbasierte Authentisierung verwendet
	# die eingebaute Authentiserung des WWW-Servers und verlaesst sich
	# auf die Umgebungsvariable, die fuer den REMOTE_USER-Namen gesetzt
	# wird.

	if ($auth_cgi eq "on")
	{
		# Die Aktionen der CGI-Authentisierung beruhen auf dem jeweils
		# vorhergehenden Bildschirm. Dies haengt davon ab, welche
		# Operationsvariable (OP) uebergeben wurde (welcher Knopf wurde
		# auf dem vorherigen Bildschirm geklickt?)

		if ($in{'auth_logon_op'} ne "")
		{
			($bad_logon_message, $session, $username, $group, @extra_fields) = 
				&CgiLogon($main_script, *in);
		} # Ende der Abfrage zu auth_logon

		if ($in{'auth_search_op'} ne "" && $auth_allow_search eq "on")
		{
			&SearchUsers($main_script, *in);
			exit;
		}

		if ($in{'auth_search_screen_op'} ne "" && $auth_allow_search eq "on")
		{
			&PrintSearchPage($main_script, *in);
			exit;
		}

		if ($in{'auth_register_op'} ne "" && $auth_allow_register eq "on")
		{
			&RegisterUser($main_script, *in);
			exit;
		} # End der Bildschirmregistrierung

		if ($in{'auth_register_screen_op'} ne "" && $auth_allow_register eq "on")
		{
			&PrintRegisterPage;
			exit;
		} # Ende des Registrierungsbildschirms

		if ($in{'auth_logon_screen_op'} ne "" || ($session eq ""))
		{
			&PrintLogonPage($bad_logon_message, $main_script, *in);
			exit;
		} # Ende des Login-Bildschirms

	} # Ende von Auth_CGI

	# Serverbasierte Authentiserung
	if ($auth_server eq "on")
	{
		$username = $ENV{'REMOTE_USER'};
		if ($username ne "")
		{
			# Das folgende ruft eine Site-spezifische, server-basierte
			# Routine auf.

			require "auth-server-lib.pl";

		} # Ende der Abfrage if($username ne "")

	}  # Ende der Abfrage if($auth_server eq "on")

	# Wenn weder cgi- noch server-basierte Authentiserungs verwendet werden,
	# wird trotzdem eine Sitzung erstellt und mit Nullstrings gefuellt.

	if ($auth_server ne "on" && $auth_cgi ne "on")
	{
		@extra_fields = ();
		$username = "";
		$group = $auth_default_group;
		foreach (@auth_extra_fields)
		{
			push (@extra_fields, "");
		}
		$session = &MakeSessionFile(@extra_fields);
	} # Ende der Abfrage weder cgi- noch server-Authentisierung

	($session, $username, $group, @extra_fields);

} # Ende der Subroutine VerifyUser

############################################################
#
# Subroutine: CGILogon
#   Verwendung:
#   ($bad_logon_message,
#    $session, $username, $group, @extra_fields) 
#    = &VerifyUser("script name", *in);
#
#  Diese Routine versucht, den Benutzer einzuloggen. Wenn das Login
#  fehlschlaegt, enthaelt meldung_kein_login den Grund dafuer. Falls
#  es erfolgreich war, enthaelt der Rest der zurueckgegebenen
#  Variablen gueltige Informationen.
#
#  Parameter:
#     $main_script = das Script, von dem aus &GetSessionInfo
#			   aufgerufen wird.
#     *in =	Ein Verweis auf die Formulardaten, die mit &ReadParse
#		eingelesen wurden.
#
#   Ausgabe:
#	$bad_logon_message = in case the logon fails
#	$session = Sitzungs-ID
#	$username = Benutzername
#	$group = Gruppeninformation
#	@extra_fields = i.d.R. ein Array weiterer Felder,
#	die aus folgendem bestehen:
#       $first_name = Vorname
#       $last_name = Nachname
#       $email = email-Adresse
############################################################

sub CgiLogon
{
	local ($main_script, *in) = @_;
	local($password, $username, $group, @extra_fields);
	local($form_password, $session, @fields);
	local($bad_logon_message);
	local($user_matched);

	$bad_logon_message = "";
	$session = "";
	@fields = ();
	$form_password = $in{"auth_password"};
	open (USERFILE, "$auth_user_file");

	$user_matched = 0;
	while (<USERFILE>)
	{
		chop($_);
		($password, $username, $group, $first_name, $last_name, $email) = split(/\|/, $_);
		if ($in{'auth_user_name'} eq $username)
		{
			$user_matched = 1;
			if (&AuthEncryptWrap ($form_password, $password) eq $password)
			{
				@fields = split(/\|/, $_);
				shift(@fields); # Kennwortfeld loswerden
				$session = &MakeSessionFile(@fields);
			} # Ende der Abfrage, ob das Kennwort stimmt
		} # Ende de Abfrage, ob der Benutzername stimmt
	} # Ende der while-schleife
	close (USERFILE);

	if ($user_matched == 1 && $session eq "")
	{
		$bad_logon_message = "<p><strong>Leider pate Ihr Kennwort nicht zum Benutzernamen.</strong><p>";
	}
	if ($user_matched == 0)
	{
		$bad_logon_message = "<p><strong>Dieser Benutzername wurde leider nicht gefunden.</strong><p>";
	}
	($bad_logon_message, $session, @fields);

} # Ende de Subroutine CgiLogon

############################################################
#
# Subroutine: MakeSessionFile 
#   Verwendung:
#   $session = &MakeSessionFile(@fields);
#
#  Diese Routine erstellt auf Grundlage der Felder, die den Benutzer
#  ausmachen (z.B. Vor- und Nachname) eine Sitzungsdatei.
#
#  Parameter:
#   @fields = Eine Liste der Felder, die den Benutzer charakterisieren
#
#   Ausgabe:
#	$session = session id
#
############################################################

sub MakeSessionFile
{
	local(@fields) = @_;
	local($session, $session_file);

	&RemoveOldSessions; 

	# Zufallsgenerator anwerfen
	srand($$|time);
	$session = int(rand(60000));

	# Zeit, Prozess-ID und die Zufallszahl werden zu einer Hexzahl vereint,
	# die die Sitzungs-ID ausmacht.

	$session = unpack("H*", pack("Nnn", time, $$, $session));

	$session_file = "$session.dat";

	open (SESSIONFILE, ">$auth_session_dir/$session_file");
	print SESSIONFILE join ("\|", @fields);
	print SESSIONFILE "\n";

	close (SESSIONFILE);

	$session;

} # Ende der Subroutine MakeSessionFile

############################################################
#
# Subroutine: RemoveOldSessions
#   Vewendung:
#     &RemoveOldSessions;
#
# Diese Routine entfernt alte Sitzungsdateine aufgrund ihres Alters
# anhand definierter Variabler.
#
#  Parameter:
#    Keine.
#
#  Ausgabe:
#     Keine.
############################################################

sub RemoveOldSessions
{
	local(@files, $file);

	# Oeffnen des Sitzungsverzeichnisses.
	opendir(SESSIONDIR, "$auth_session_dir");

	# Lesen aller eintraege ausser "." und ".."
	@files = grep(!/^\.\.?$/,readdir(SESSIONDIR));
	closedir(SESSIONDIR);

	# Alle Dateien pruefen
	foreach $file (@files)
	{
		# Falls sie aelter als auth_session_length ist,
		# wird sie geloescht.
		if (-M "$session_dir/$file" > $auth_session_length)
		{
			unlink("$auth_session_dir/$file");
		}
	}
} # Ende der Subroutine RemoveOldSessions

############################################################
#
# Subroutine: SearchUsers 
#   Verwendung:
#     &SearchUsers($main_script, *in);
#
#  Diese Routine fuehrt die Scuhe nach einem Benutzer in der
#  Benutzerdatei durch, um einem Benutzer zu erlauben, seine
#  Benutzer-ID wiederzufinden, falls sie verlorengeht.
#  Die suche basiert auf der email-Adresse.
#
#  Parameter:
# 	$main_script = das Script, von dem aus &GetSessionInfo
#			   aufgerufen wird.
#     *in =	Ein Verweis auf die Formulardaten, die mit &ReadParse
#		eingelesen wurden.
#
#   Ausgabe:
#     HTML-Ausgabe mit einem Hinweis auf den Fehler oder im Falle
#     des Erfolges die Anzeigedes Benutzernamens, welcher der
#     der email-adresse entsprach.
############################################################

sub SearchUsers
{
	local($main_script, *in) = @_;
	local($user_matched);
	local($field_num, $username, @extra_fields);
	local($auth_email);
	local($user_list);

	local($form_tags);

	$form_tags = &PrintCurrentFormVars(*in);
	open (USERFILE, "$auth_user_file");

	$auth_email = $in{'auth_email'};
	$auth_email =~ tr/A-Z/a-z/;
	$auth_email =~ s/ //g;
	$user_match = "";

	$field_num = 0;
	foreach (@auth_extra_fields)
	{
		if ($_ eq "auth_email")
		{
			last;
		}
		$field_num++;
	}

	while (<USERFILE>)
	{
		chop($_);
		tr/A-Z/a-z/;
		@extra_fields = split(/\|/, $_);
		$username = $extra_fields[1];

		# Loeschen der ersten drei Felder
		shift(@extra_fields);	
		shift(@extra_fields);
		shift(@extra_fields);
		$email =~ s/ //g;
		if ($auth_email eq $extra_fields[$field_num])
		{
			$user_match .= $username . "|";
		} # Ende der Abfrage nach uebereinstimnmender email-Adresse
	} # Ende der while-Schleife
	close (USERFILE);
	if ($user_match ne "")
	{
		$user_list = $user_match;
		$user_list =~ s/\|/<p>/g;
	} 

	if ($user_match eq "")
	{
		&HTMLPrintNoSearchResults($main_script, $form_tags);
	}
	else
	{
		&HTMLPrintSearchResults ($main_script, $form_tags, $user_list);
	} # Ende der Abfrage if ($user_match eq "")

} # Ende der Subroutine SearchUsers

############################################################
#
# Subroutine: RegisterUser 
#   Verwendung:
#     &RegisterUser($main_script, *in);
#
# Diese Routine registriert den Benutzer und fuehrt etliche
# Ueberpruefungen durch, bevor der Benutzer schliesslich in
# Benutzerdatei uebernommen wird und der Systemadministrator
# ggf. ein email erhaelt.
#
#  Parameter:
#	$main_script = the script you are calling
#                    &GetSessionInfo From
#     *in = A reference to the form data that was read
#           in with &ReadParse.
#
#   Ausgabe:
#     HTML-Ausgabe bzl. erfolgreicher oder fehlgeschlagener
#     Registrierung.
############################################################

sub RegisterUser
{
	local($main_script, *in) = @_;
	local($x, @form_vars, $group, @f);
	local(@write_fields, $password);
	local($form_tags, $user_matched);
	local($userfile,$user_email,$real_password);
	local($random,$salt);
	$form_tags = &PrintCurrentFormVars(*in);
	$group = $auth_default_group;

	# Diese Variablen muessen einen Wert enthalten, und sie
	# duerfen keinen senkrechten Strich enthalten - ansonsten
	# werden sie nicht akzeptiert.
	@form_vars = ('auth_user_name', @auth_extra_fields);

	if ($auth_generate_password ne "on")
	{
		push(@form_vars, "auth_password1");
		push(@form_vars, "auth_password2");
	}

	foreach $x (@form_vars)
	{
		if ($in{"$x"} eq "" || $in{"$x"} =~ /\|/)
		{
			&HTMLPrintRegisterNoValidValues($main_script, $form_tags);
			exit;
		} # Ende der Abfrage nach ungueltigen/fehlenden Inhalten

		if ($x eq "auth_email")
		{
			$user_email = $in{"$x"};
		}
	} # Ende der foreach-Schleife

	# Die eigegebenen Kennwoerter muessen uebereinstimmen
	if ($in{'auth_password1'} ne $in{'auth_password2'})
	{
		&HTMLPrintRegisterNoPasswordMatch($main_script, $form_tags);
		exit;
	}

	# Ueberpruefung auf doppelte Namen, falls $auth_check_duplicates
	# auf "on" steht.

	if ($auth_check_duplicates eq "on")
	{
		open (USERFILE, "$auth_user_file");
		$user_matched = 0;
		while (<USERFILE>)
		{
			chop($_);
			@f = split(/\|/, $_);
			if ($f[1] eq $in{'auth_user_name'})
			{
				$user_matched = 1;
				last;
			}
		} # Ende der while-Schleife
		close (USERFILE);

		# Ueberpruefen auf Duplikate in der alternativen Benutzerdatei,
		# falls sie definiert ist.
		if ($auth_alt_user_file ne "")
		{
			open (USERFILE, "$auth_alt_user_file");

			while (<USERFILE>)
			{
				chop($_);
				@f = split(/\|/, $_);
				if ($f[1] eq $in{'auth_user_name'})
				{
					$user_matched = 1;
					last;
				}
			} # Ende der while-Schleife
			close (USERFILE);
		} # Ende der Abfrage der alternativen Datenbank

		if ($user_matched == 1)
		{
			&HTMLPrintRegisterFoundDuplicate($main_script, $form_tags);
			exit;
		} # Ende der Abfrage if ($user matched == 1) (Doppeleintrag gefunden)
	} # Ende der Abfrage if ($auth_check_duplicates eq "on")

	# Erzeugen des Kennworts, falls es nicht eingegeben worden ist.
	# Verschluesseln des Kennworts und Erzeugen des Datensatzes, der
	# in die Benutzerdatei geschrieben oder per email an den Admini-
	# strator geschickt wird.

	# Anwerfen des Zufallsgenerators
	srand(time|$$);

	if ($auth_generate_password ne "on")
	{
		$password = $in{"auth_password1"};
	}
	else
	{
		$random = "abcdefghijklmnopqrstuvwxyz1234567890";
		$password = "";
		for (1..6)
		{
			$password .= substr($random,int(rand(36)),1);
		}
	}
	$real_password = $password;
	$salt = "";

	# Anmerkung: Der Zufallsgenerator wird nicht erneut angeworfen.
	$random = "abcdefghijklmnopqrstuvwxyz1234567890";
	for (1..2)
	{
		$salt .= substr($random,int(rand(36)),1);
	}
	$password = &AuthEncryptWrap ($password, $salt);
	@write_fields = ($password, $in{'auth_user_name'}, $group);
	foreach (@auth_extra_fields)
	{
		push(@write_fields, $in{"$_"});
	}

	# Hinzufuegen des Benutzers zur Benutzerdatei
	if ($auth_add_register eq "on")
	{
		# Sperren der Datei, so dass niemand sonst gleichzeitig darauf zugreift
		# (falls parallel eine weitere Registrierung stattfindet).

		&AuthGetFileLock ("$auth_user_file.lock");
		$user_file = $auth_user_file;
		if ($auth_alt_user_file ne "")
		{
			$user_file = $auth_alt_user_file;
		}
		open (USERFILE, ">>$user_file");

		print USERFILE join("\|", @write_fields) . "\n";
		close (USERFILE);
		&AuthReleaseFileLock ("$auth_user_file.lock");

	} # Ende der Abfrage if ($auth_add_register eq "on")

	# Nun wird die Registrierungs-Information an den Administrator gemailt,
	# falls diese Option aktiviert wurde.

	if ($auth_email_register eq "on" 
	&& $auth_admin_from_address ne "" 
	&& $auth_admin_email_address ne "")
	{
		require "$auth_lib/mail-lib.pl";
		local($subject, $message);
		$subject = "Benutzerregistrierung";

		$message = join(",", @write_fields) . "\n";
		&send_mail($auth_admin_from_address, $auth_admin_email_address, $subject, $message);

	} # Ende der Abfrage zur Email-Benachrichtigung

	# Emailen des Kennworts, falls es automatisch generiert wurde.

	if ($auth_generate_password eq "on")
	{
		if ($auth_email_register ne "on")
		{
			require "$auth_lib/mail-lib.pl";
		}
		$subject = "Kennwort fr Benutzerregistrierung";
		$message = $auth_password_message;
		$message .= " $real_password.\n\n\n";
		&send_mail($auth_admin_from_address, $user_email, $subject, $message);
	} # Ende der Abfrage, ob das Kennwort automatisch erzeugt wurde
	&HTMLPrintRegisterSuccess ($main_script, $form_tags);

} # Ende der Subroutine RegisterUser

############################################################
#
# Subroutine: PrintCurrentFormVars
#   Verwendung:
#     $form_tags = &PrintCurrentFormVars(*formvars);
#
#  Diese Routine nimmt die vorhandenen Formularvariablen und erzeugt aus
#  ihnen verborgenen Variable, die in das neue Formular geschrieben werden.
#  Auf diese Weise werden alte Formularvariable intakt gehalten, waehrend
#  die ganze Palette an cgi-Formularen um die Registrierung herum abgear-
#  beitet wird.
#  Alle Variablen, in denen AUTH vorkommt, werden nicht dupliziert, da sie
#  mit diesem Script zusammenhaengen.
#
#  Parameter:
#    *form_vars = Ein Verweis auf die mit dem Hauptscript verknuepften
#    Formularvariablen, von dem aus die Authentiserungsroutinen zuerst
#    aufgerufen wruden.
#
#   Ausgabe:
#     Verborgene Formularfelder, die mit weiterhin zu speichernden
#     Formulardten zusammenhaengen.
############################################################

sub PrintCurrentFormVars
{
	local(*form_vars) = @_;
	local($x,$y,$form_tags);
	$form_tags = "";
	foreach $x (keys %form_vars)
	{
		if (!($x =~ /auth_/i))
		{
			$y = $form_vars{"$x"};
			$form_tags .= qq!<input type="hidden" name="$x" value="$y">\n!;
		}
	}
	$form_tags;
} # Ende de Subroutine PrintCurrentFormTags

############################################################
#
# Subroutine: AuthGetFileLock 
#   Verwendung:
#     &AuthGetFileLock("lockfilename");
# 
#  Diese Routine sperrt eine Datei, so dass niemand sonst darauf
#  zugreifen kann, waehrend in sie geschrieben wird.
# 
#  Parameter:
#     $lock_file = Dateiname der temporaeren Sperrdatei
#
#   Ausgabe:
#      Keine. Das Vorhandensein der Sperrdatei wird ueberprueft. Falls
#      sie gefunden wird, wird gewartet, bis die Sperrdatei verschwindet.
#      Danach oeffnet die Routine selbst eine Sperrdatei.
#
#   Anmerkung: FLOCK wird nicht verwendet, weil es auf einigen Systemen
#   nicht implementiert ist. Falls es auf Ihrem System laeuft, koennen
#   Sie die FLOCK-Zeilen entkommentieren.
############################################################

sub AuthGetFileLock
{  
	local ($lock_file) = @_;

	local ($endtime);  
	$endtime = 60;
	$endtime = time + $endtime;

	# Nach 60 Sekunden muss irgendein anderer Fehler vorliegen (vorheriger Absturz z.B.)

	# Die Variable $endtime bestimmt die Wartezeit, falls ein anderer Aufruf dieses
	# Scripts die Sperrdatei bereits geoeffnet hat.
	while (-e $lock_file && time < $endtime)
	{
		# 1 Sekunde warten
		sleep(1);
	}
	open(LOCK_FILE, ">$lock_file");    
	# flock(LOCK_FILE, 2); # 2 sperrt die Datei fuer exklusiven Zugriff
} # Ende der Subroutine AuthGetFileLock

############################################################
#
# Subroutine: AuthReleaseFileLock
#   Verwendung:
#     &AuthReleaseFileLock($lock_file);
#
#  Diese Routine gibt die Sperrdatei wieder frei, wenn der Schreibvorgang in die
#  Originaldatei beendet ist.
#
#     $lock_file = Dateiname der temporaeren Sperrdatei
#
#   Ausgabe:
#      Keine. die Sperrdatei wird entfernt.
# 
#   Anmerkung: FLOCK wird nicht verwendet, weil es auf einigen Systemen
#   nicht implementiert ist. Falls es auf Ihrem System laeuft, koennen
#   Sie die FLOCK-Zeilen entkommentieren.
############################################################

sub AuthReleaseFileLock
{
	local ($lock_file) = @_;
       
	# flock(LOCK_FILE, 8); # 8 entsperrt die Datei

	close(LOCK_FILE);
	unlink($lock_file);

} # Ende der Subroutine ReleaseFileLock   
 
############################################################
#
# Subroutine: AuthEncryptWrap
#   Verwendung:
#     $cryptedpassword = &AuthEncryptWrap($password, $salt);
#
#  Diese Routine kleidet die CRYPT-Routine in eine weitere ein, so dass
#  Sie Ihre eigene implementieren koenne, falls Ihr System CRYPT nicht
#  unterstuertzt.
#
#  Parameter:
#    $field = zu verschluesselndes Feld
#    $salt = fuer die Verschluesselung zu verwendender salt-Wert
#
#   Ausgabe:
#     Verschluesseltes Kennwort
############################################################

sub AuthEncryptWrap
{
	local ($field, $salt) = @_;
 
	$field = crypt ($field, $salt);
 
	$field;
 
} # Ende der Subroutine encrypt
 
1;
