############################################################
#                       SENDMAIL_LIB.PL
#
# Dieses Script wurde von Gunther Birznieks geschrieben. 
# Urspruengliche Programmierung:	22.02.96
# Letzte Aenderung:			05.05.96
# Deutsche Uebersetzung: Gerold Bigorajski
#
#   Es darf unter den Bedingungen der GNU General Public
#   License oder der Artistic License, die zusammen mit
#   Perl v5.x fr UNIX verteilt wird, kopiert werden.
# 
# Gerold Bigorajski ist unter bigorajski@t-online.de erreichbar.
#
# Zweck: Es stellt eine Reihe von Bibliotheksroutinen zur Verfuegung,
# um email ueber das Internet zu verschicken.
#
# Hauptprozeduren:
#  real_send_mail - flexible Art und Weise zur Versendung von emails
#  send_mail - einfacher zu verwendende Version von send_mail
#
# Spezielle Anmerkungen: Das Script ist UNIX-spezifisch und mit dem
# Sendmail-Program verknuepft, das i.d.R. im Verzeichnis /usr/lib/sendmail
# oder /usr/sbin/sendmail gespeichert ist.
# 
# Denken Sie auf daran, dass dem Klammeraffen ("@") ein Backslash ("\")
# vorangestellt werden muss, damit er in PERL 5 nicht als Steuerzeichen
# interpretiert wird.
#
# Aendern Sie die Variable $mail_program auf den Speicherort Ihres
# Sendmail-Programms.
#
############################################################

# $mail_program ist das Mail-Programm, das zum Versenden von Mail
# benutzt wird -  mit voller Pfadangabe.  
#
# Der Flag -t weist sendmail an, in den Kopfzeilen des Mails nach
# To:, From:, Subject: Ausschau zu halten, um die Empfaenger-
# Adresse zu bestimmen.
#
# Der Flag -n instruiert sendmail, die Alias-Liste fuer den  UNIX-
# Server nicht zu verwenden. In den meisten Faellen ist es unerwuenscht,
# dass Aussenstehende auf die Aliasliste des Webservers zugreifen.

$mail_program = "/usr/sbin/sendmail -t -n";

############################################################
#
# Subroutine: real_send_mail 
#   Verwendung:
#   Verwendung:
#     &send_mail("ich@meine_domain.de","meine_domain.de","du@dein_provider.de",
#     "dein_provider.de", "MeinBetreff", "Meine Mitteilung");
#
#   Parameter:
#     $fromuser = Vollstaendige Email-Adresse des Absenders
#     $fromsmtp = Vollstaendige Internetadresse des SMTP-Servers des Absenders
#     $touser   = Vollstaendige Email-Address des Empfaengers
#     $tosmtp   = Vollstaendige Internetadresse des SMTP-Servers des Empfaengers
#     $subject  = Betreff der Mitteilung
#     $messagebody = Inhalt der Mitteilung einschliesslich Zeilenumbuechen.
#
#   Ausgabe:
#     keine
############################################################

sub real_send_mail
{
	local($fromuser, $fromsmtp, $touser, $tosmtp, $subject, $messagebody) = @_;

	# Als erstes wird das Mail-Programm gestartet und eine PIPE zu dem Programm
	# geoeffnet, so dass alles, was in den Filehandle ausgegeben wird beim
	# laufenden Progrmm landet.
	open (MAIL, "|$mail_program") || &web_error("Konnte Mail-Program nicht oeffnen");

	# Ausgabe der Mitteilung an das Mailprogramm
	# unter Verwendung der Hier-folgt-ein-Dokument-Methode
	# Anmerkung: Die abschliessende Marke __END_OF_MAIL__ 
	# MUSS am Zeilenanfang stehen!

	print MAIL <<__END_OF_MAIL__;
To: $touser
From: $fromuser
Subject: $subject

$messagebody

__END_OF_MAIL__

	close (MAIL);

} #End der Subroutine real_send_mail

############################################################
#
# Subroutine: send_mail 
#   Verwendung:
#     &send_mail("ich@meine_domain.de","du@dein_provider.de",
#     "MeinBetreff", "Meine Mitteilung");
#
#   Parameter:
#     $fromuser = Vollstaendige Email-Adresse des Absenders
#     $touser   = Vollstaendige Email-Address des Empfaengers
#     $subject  = Betreff der Mitteilung
#     $messagebody = Inhalt der Mitteilung einschliesslich Zeilenumbuechen.
#
#   Ausgabe:
#     keine
#
############################################################

sub send_mail
{
	local($from, $to, $subject, $messagebody) = @_;

	local($fromuser, $fromsmtp, $touser, $tosmtp);

	# Diese Routine nimmt die einfacheren Parameter von send_mail
	# undsplittet sie in diejenigen auf, die an real_send_mail
	# uebergeben werden muessen.

	$fromuser = $from;
	$touser = $to;

	# Split wird verwendet, um die Adresse in Paare Benutzer/Hostname
	# aufzusplitten. Der Hostname ist das zweite Element des
	# aufgesplitteten Arrays, er hat daher den Index 1 (Arrays
	# beginnen mit Index 0.

	$fromsmtp = (split(/\@/,$from))[1];
	$tosmtp = (split(/\@/,$to))[1];

	# Nun wird die Sendmail-Routine mit den neu erzeugten Parametern
	# aufgerufen.

	&real_send_mail($fromuser, $fromsmtp, $touser, $tosmtp, $subject, $messagebody);

} # Ende der Subroutine send_mail

############################################################
#
# Subroutine: web_error
#   Verwendung:
#     &web_error("Datei xxx konnte nicht geffnet werden.");
#
#   Parameter:
#     $error = Beschreibung des Fehlers
#
#   Ausgabe:
#     keine
# 
############################################################

sub web_error
{
	local ($error) = @_;
	$error = "Es ist ein Fehler aufgetreten: $error";
	print "$error<p>\n";

	# Die beendet das Programm vorzeitig und gibt eine
	# Fehlermeldung an stderr aus.

	die $error;

} # Ende der Subroutine web_error

1;
